//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//		BAlert-like window for the about box.

#include <Application.h>
#include <Message.h>
#include <OS.h>
#include <Point.h>
#include <Rect.h>
#include <View.h>
#include <Window.h>

#include "SamhainWindow.h"

const BPoint kDefaultLocation(100, 100);

enum {
	kButton1Pressed = 'btn1',
	kButton2Pressed = 'btn2'
};

static BRect
calcFrame(BRect viewBounds)
{
	viewBounds.OffsetBy(kDefaultLocation);
	return viewBounds;
}

SamhainWindow::SamhainWindow(
	BView *pView, const char *pLabel1, const char *pLabel2)
:	inherited(calcFrame(pView->Bounds()), "Samhain",
				 B_MODAL_WINDOW,
				 B_NOT_RESIZABLE|B_NOT_CLOSABLE|B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE),
	mButtonSem(B_BAD_SEM_ID),
	mWhichButton(-1)
{
	AddChild(pView);
	
	// Create the buttons.
	const BRect emptyFrame(B_ORIGIN, B_ORIGIN);

	BButton *pButton1 =
		new BButton(emptyFrame, "button1", pLabel1, new BMessage(kButton1Pressed));
	pButton1->SetTarget(this);
	pButton1->ResizeToPreferred();
	float width1, height1;
	pButton1->GetPreferredSize(&width1, &height1);

	BButton *pButton2 =
		new BButton(emptyFrame, "button2", pLabel2, new BMessage(kButton2Pressed));
	pButton2->SetTarget(this);
	pButton2->ResizeToPreferred();
	float width2, height2;
	pButton2->GetPreferredSize(&width2, &height2);

	// Size & place the buttons.
	const float width = (width1 > width2) ? width1 : width2;
	const float height = (height1 > height2) ? height1 : height2;
	pButton1->ResizeTo(width, height);
	pButton2->ResizeTo(width, height);

	const BRect bounds(pView->Bounds());
	pButton2->MoveTo(bounds.RightBottom() - BPoint(width + 0.5*height, 1.5*height));
	pView->AddChild(pButton2);
	pButton1->MoveTo(bounds.RightBottom() - BPoint(2.0*width + height, 1.5*height));
	pView->AddChild(pButton1);
	SetDefaultButton(pButton2);
}

SamhainWindow::~SamhainWindow(void)
{
	if (mButtonSem >= B_NO_ERROR) {
		delete_sem(mButtonSem);
	}
}

void
SamhainWindow::MessageReceived(BMessage *pMessage)
{
	switch (pMessage->what) {
	
	case kButton1Pressed:
	case kButton2Pressed:
		// Button pressed.
		mWhichButton = pMessage->what - kButton1Pressed;
		if (mButtonSem >= B_NO_ERROR) {
			release_sem(mButtonSem);
		}
		break;
	
	default:
		inherited::MessageReceived(pMessage);
		break;
	}
}

bool
SamhainWindow::QuitRequested(void)
{
	if (mButtonSem >= B_NO_ERROR) {
		release_sem(mButtonSem);
	}
	return true;
}

int32
SamhainWindow::Go(void)
{
	Show();

	int32 result = -1;	
	if (Lock()) {
		mButtonSem = create_sem(0, "SamhainWindow button sem");
		Unlock();
		acquire_sem(mButtonSem);
		result = mWhichButton;
	}

	if (Lock()) {
		Quit();
	}
	
	return result;
}
